@@ -1,5 +1,5 @@
 ## Copyright (C) 2005 Carl Osterwisch <osterwischc@asme.org>
-## Copyright (C) 2013 Carnë Draug <carandraug@octave.org>
+## Copyright (C) 2013-2019 Carnë Draug <carandraug@octave.org>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -47,9 +47,9 @@ function y = units (fromUnit, toUnit, x = 1)
     error ("units: X must be numeric");
   endif
 
-  persistent available  = check_units ();
-  persistent compact    = has_compact_option ();
-  persistent template   = template_cmd (compact);
+  persistent cmd        = check_units ();
+  persistent compact    = has_compact_option (cmd);
+  persistent template   = template_cmd (cmd, compact);
 
   ## We have to insert the template on the string this way, because it may have
   ## a %%.16g which we may want to keep for later use in non-linear conversion
@@ -105,40 +105,65 @@ function y = units (fromUnit, toUnit, x = 1)
   endif
 endfunction
 
-function fpath = check_units ()
+function name = check_units ()
   ## See bug #38270 about why we're checking this way.
-  fpath = file_in_path (getenv ("PATH"), sprintf ("units%s", octave_config_info ("EXEEXT")));
+  ## Also try use gunits over units
+  name = "gunits";
+  fpath = file_in_path (getenv ("PATH"), sprintf ("gunits%s", get_exeext()));
   if (isempty (fpath))
+    name = "units";
+    fpath = file_in_path (getenv ("PATH"), sprintf ("units%s", get_exeext()));
+  endif
+  if (isempty (fpath))
+    name = "";
     error ("units: %s\nVerify that GNU units is installed in the current path.",
            rawoutput);
   endif
 endfunction
 
-function compact = has_compact_option ()
+function compact = has_compact_option (cmd)
   compact = true;
   ## We must give some units to convert because the only thing that would
   ## make it not do any work (--version) actually exits with exit value 3
-  [status, rawoutput] = system ('units --compact --one-line "in" "cm"');
+  #
+  is_dos = (ispc() && !isunix());
+  if is_dos
+    redirect = '2> NUL';
+  else
+    redirect = '2> /dev/null';
+  endif
+ 
+  [status, rawoutput] = system ([cmd ' --compact --one-line "in" "cm" ' redirect]);
   if (status)
     compact = false;
   endif
 endfunction
 
-function template = template_cmd (compact)
+function template = template_cmd (cmd, compact)
   ## do we have the format option?
   format = true;
-  [status, rawoutput] = system ('units --output-format "%.16g" "in" "cm"');
+
+  is_dos = (ispc() && !isunix());
+  if is_dos
+    redirect = '2> NUL';
+  else
+    redirect = '2> /dev/null';
+  endif
+ 
+  [status, rawoutput] = system ([cmd ' --output-format "%.16g" "in" "cm" ' redirect]);
   if (status)
     format = false;
   endif
 
-  template = "units ";
+  template = [cmd " "];
   if (format)
     template = [template '--output-format "%%.16g" '];
   endif
   if (compact)
     template = [template '--compact --one-line '];
   endif
+
+  template = [template ' ' redirect ' '];
 endfunction
 
 ## Test the correct way to do non-linear conversion.
